# Stanford::LSDB -- Wrapper module for LS database connections.
# $Id: LSDB.pm,v 1.5 2004/10/05 00:14:15 eagle Exp $
#
# Written by Russ Allbery <rra@stanford.edu>
# Copyright 2001, 2004 Board of Trustees, Leland Stanford Jr. University
#
# This program is free software; you may redistribute it and/or modify it
# under the same terms as Perl itself.
#
# This module is a thin wrapper around DBI using the DBD::Oracle driver to
# make it easier to talk to one of the Leland Systems databases.  It's
# intended to be subclassed for each database that we have, so that things
# like usernames and passwords don't have to be coded into any script.

##############################################################################
# Modules and declarations
##############################################################################

# Set up the subclasses.  This is required to avoid warnings under DBI 1.40
# and later, even though we don't actually make use of any overridden
# statement handle or database handle methods.
package Stanford::LSDB::st;
use vars qw(@ISA);
@ISA = qw(DBI::st);

package Stanford::LSDB::db;
use vars qw(@ISA);
@ISA = qw(DBI::db);

package Stanford::LSDB;

require 5.005;

use Carp;
use DBI;

use strict;
use vars qw(@ISA $VERSION);

# Inherit from DBI so that our objects can be used as DBI objects.
@ISA = qw(DBI);

# The version of this module is its CVS revision.
($VERSION = (split (' ', q$Revision: 1.5 $ ))[1]) =~ s/\.(\d)$/.0$1/;

##############################################################################
# Core overrides
##############################################################################

# Override DBI::connect to supply our own connect string, username, and
# password and to set some standard options.  Subclasses should override this
# method to supply the hostname, SID, username, and password.  Expects a
# connect string of the form hostname:SID.
sub connect {
    my ($class, $connect, $user, $passwd, @extra) = @_;
    $ENV{ORACLE_HOME} = '/usr/pubsw/apps/oracle/8.1.7';
    my ($hostname, $sid) = split (':', $connect);
    Carp::croak ("Bad connect string $connect (should be host:SID)")
        unless $sid;
    my $source = "dbi:Oracle:host=$hostname;sid=$sid";
    my %attrs = (PrintError => 0, RaiseError => 1, AutoCommit => 1, @extra);
    $class->SUPER::connect ($source, $user, $passwd, \%attrs);
}

##############################################################################
# New methods
##############################################################################

# Given a standard set of arguments for connect except with a filename rather
# than a literal password, load the password from the file and pass things off
# to the main connect sub.
sub connect_file {
    my ($class, $connect, $user, $passfile, @extra) = @_;
    my $passwd = $class->_read_password ($passfile);
    Stanford::LSDB::connect ($class, $connect, $user, $passwd, @extra);
}

# Provide an interface to running sqlldr so that scripts don't need to know
# where it is and the syntax for it.  Takes the username, the password file,
# the control file, the log file, and any additional key/value pairs that the
# user wants to pass to sqlldr.  Returns 0 on success, the return status of
# sqlldr on failure (defined by the return status of sqlldr).
sub sqlldr {
    my ($class, $user, $passfile, $control, $log, @extra) = @_;
    my $passwd = $class->_read_password ($passfile);
    local $ENV{ORACLE_HOME} = '/u01/app/oracle/OraHome1';
    my $status = system ('/u01/app/oracle/OraHome1/bin/sqlldr',
                         "userid=$user/$passwd", "control=$control",
                         "log=$log", 'silent=(header,feedback)');
    return ($status == 0 ? 0 : $status >> 8);
}

##############################################################################
# Private methods
##############################################################################

# Read a password from a file.  Assumes that the file contains the password on
# the first line, terminated by a newline.
sub _read_password {
    my ($class, $file) = @_;
    open (PASSWD, $file) or die "Cannot open $file: $!";
    my $passwd = <PASSWD>;
    close PASSWD;
    chomp $passwd;
    return $passwd;
}

##############################################################################
# Module return value and documentation
##############################################################################

# Make sure the module returns true.
1;

__END__

=head1 NAME

Stanford::LSDB - Wrapper module for LS database connections

=head1 SYNOPSIS

    use Stanford::LSDB;

    $dbh = Stanford::LSDB->connect ("$host:$sid", $user, $passwd);

    $dbh = Stanford::LSDB->connect_file ("$host:$sid", $user, $file);

=head1 DESCRIPTION

This is a small wrapper module around the DBI and DBD::Oracle modules that
takes care of building a connect string.  It's meant to be subclassed by
other modules to provide a username, password, host, and SID so that the
user doesn't even need to know that.

The only overrided method is connect().  All other methods should work the
same as DBI.  connect() will automatically set the PrintError option to
false and the RaiseError and AutoCommit options to true.  (These can, of
course, be changed later.)

This module also provides the following additional methods:

=over 4

=item connect_file (I<connect-string>, I<username>, I<file> [, I<extra> ...])

Takes the same arguments as connect(), except that the password provided is
instead a file name, which will be opened and read to obtain the file.
Assumes the password is on the first line of the file, terminated by a
newline.

=item sqlldr (I<user>, I<file>, I<control>, I<log> [, I<extra> ...])

Runs Oracle's B<sqlldr> command with the provided control and log files,
connecting with the given username and reading the password from the
provided file.  Returns 0 on success and the exit status of B<sqlldr> on
failure, where success is defined as B<sqlldr> exiting with status 0.

=back

=head1 AUTHOR

Russ Allbery <rra@stanford.edu>

=head1 SEE ALSO

L<DBI|DBI>, L<DBD::Oracle|DBD::Oracle>

=head1 COPYRIGHT AND LICENSE

Copyright 2001, 2004 Board of Trustees, Leland Stanford Jr. University.  This
program is free software; you may redistribute it and/or modify it under the
same terms as Perl itself.

=cut
