# Stanford::LSDB::AFSDB -- Wrapper module for LS AFS database connections.
# $Id: AFSDB.pm,v 1.5 2004/04/21 03:56:34 eagle Exp $
#
# Written by Russ Allbery <rra@stanford.edu>
# Copyright 2001 Board of Trustees, Leland Stanford Jr. University
#
# This program is free software; you may redistribute it and/or modify it
# under the same terms as Perl itself.
#
# This module subclasses Stanford::LSDB to talk to the Leland Systems AFS
# database, allowing the user to not bother passing any options to connect.

##############################################################################
# Modules and declarations
##############################################################################

# Set up the subclasses.  This is required to avoid warnings under DBI 1.40
# and later, even though we don't actually make use of any overridden
# statement handle or database handle methods.
package Stanford::LSDB::AFSDB::st;
use vars qw(@ISA);
@ISA = qw(Stanford::LSDB::st);

package Stanford::LSDB::AFSDB::db;
use vars qw(@ISA);
@ISA = qw(Stanford::LSDB::db);

package Stanford::LSDB::AFSDB;

require 5.005;

use Stanford::LSDB;

use strict;
use vars qw(@ISA $VERSION);

@ISA = qw(Stanford::LSDB);

# The version of this module is its CVS revision.
($VERSION = (split (' ', q$Revision: 1.5 $ ))[1]) =~ s/\.(\d)$/.0$1/;

##############################################################################
# Core overrides
##############################################################################

# Override Stanford::LSDB::connect to pass in the right connection string,
# username, and password.  Pick up any extra parameters and pass them along to
# our parent object untouched.
sub connect {
    my ($class, @extra) = @_;
    my $connect = 'dbi:Oracle:host=lsdb.stanford.edu;sid=LSDB';
    $class->SUPER::connect_file ('lsdb.stanford.edu:LSDB', 'afsdb',
                                 '/db/passwords/afsdb', @extra);
}

##############################################################################
# Additional functions
##############################################################################

# Provide an interface to running sqlldr so that scripts don't need to know
# what username and password they need to use.  Takes the control file, the
# log file, and any additional key/value pairs that the user wants to pass to
# sqlldr.  Returns 0 on success, the return status of sqlldr on failure
# (defined by the return status of sqlldr).
sub sqlldr {
    my ($class, @rest) = @_;
    local $ENV{TWO_TASK} = 'LSDB';
    return $class->SUPER::sqlldr ('afsdb', '/db/passwords/afsdb', @rest);
}

##############################################################################
# Module return value and documentation#
#############################################################################

# Make sure the module returns true.
1;

__END__

=head1 NAME

Stanford::LSDB::AFSDB - Wrapper module for LS AFS database connections

=head1 SYNOPSIS

    use Stanford::LSDB::AFSDB;

    $dbh = Stanford::LSDB::AFSDB->connect;

=head1 DESCRIPTION

This is a subclass of Stanford::LSDB, a small wrapper module around the DBI
and DBD::Oracle modules.  It overrides connect() so that no arguments are
necessary; the connect string, username, and password are determined by the
module.

All other methods work the same as with any DBI object.

This module also provides the following additional methods:

=over 4

=item sqlldr (I<control>, I<log> [, I<extra> ...])

Runs Oracle's B<sqlldr> command with the provided control and log files,
connecting to the AFS database with the appropriate username and password.
Returns 0 on success and the exit status of B<sqlldr> on failure, where
success is defined as B<sqlldr> exiting with status 0.

=back

=head1 AUTHOR

Russ Allbery <rra@stanford.edu>

=head1 SEE ALSO

L<DBI|DBI>, L<DBD::Oracle|DBD::Oracle>, L<Stanford::LSDB|Stanford::LSDB>

=head1 COPYRIGHT AND LICENSE

Copyright 2001 Board of Trustees, Leland Stanford Jr. University.  This
program is free software; you may redistribute it and/or modify it under the
same terms as Perl itself.

=cut
