# Stanford::LSDB::UserDB -- Wrapper module for LS user database connections.
# $Id: UserDB.pm,v 1.5 2004/10/09 01:03:48 eagle Exp $
#
# Written by Russ Allbery <rra@stanford.edu>
# Copyright 2002 Board of Trustees, Leland Stanford Jr. University
#
# This program is free software; you may redistribute it and/or modify it
# under the same terms as Perl itself.
#
# This module subclasses Stanford::LSDB to talk to the Leland Systems AFS
# database, allowing the user to not bother passing any options to connect.

##############################################################################
# Modules and declarations
##############################################################################

package Stanford::LSDB::UserDB;

require 5.005;

use strict;
use vars qw(@ISA $VERSION);

use Stanford::LSDB;

@ISA = qw(Stanford::LSDB);

# The version of this module is its CVS revision.
($VERSION = (split (' ', q$Revision: 1.5 $ ))[1]) =~ s/\.(\d)$/.0$1/;

##############################################################################
# Core overrides
##############################################################################

# Override Stanford::LSDB::connect to pass in the right connection string,
# username, and password.  Pick up any extra parameters and pass them along to
# our parent object untouched.
sub connect {
    my ($class, @extra) = @_;
    my $connect = 'dbi:Oracle:host=lsdb.stanford.edu;sid=LSDB';
    $class->SUPER::connect_file ('lsdb.stanford.edu:LSDB', 'userdb',
                                 '/db/passwords/userdb', @extra);
}

##############################################################################
# Additional functions
##############################################################################

# Provide an interface to running sqlldr so that scripts don't need to know
# what username and password they need to use.  Takes the control file, the
# log file, and any additional key/value pairs that the user wants to pass to
# sqlldr.  Returns 0 on success, the return status of sqlldr on failure
# (defined by the return status of sqlldr).
sub sqlldr {
    my ($class, @rest) = @_;
    local $ENV{TWO_TASK} = 'LSDB';
    return $class->SUPER::sqlldr ('userdb', '/db/passwords/userdb', @rest);
}

##############################################################################
# New database handle methods
##############################################################################

# Set up the subclasses.  This is required to avoid warnings under DBI 1.40
# and later.
package Stanford::LSDB::UserDB::st;
use vars qw(@ISA);
@ISA = qw(Stanford::LSDB::st);

package Stanford::LSDB::UserDB::db;
use vars qw(@ISA);
@ISA = qw(Stanford::LSDB::db);

use Carp qw(croak);
use POSIX qw(strftime);

# Log a message to history with the current date.  Takes the principal
# affected, the message, and optionally the person making the change (which is
# appended to the history message if given).  The timestamp on the history
# entry will be the current time.
sub history_log {
    my ($self, $principal, $comment, $actor) = @_;
    my $date = strftime ('%Y-%m-%d %T', localtime);
    $comment .= " by $actor" if $actor;
    my $sql = "insert into history (hi_principal, hi_comment, hi_date)
        values (?, ?, to_date (?, 'YYYY-MM-DD HH24:MI:SS'))";
    $self->do ($sql, undef, $principal, $comment, $date);
}

# Given an account and a service, check to see if that account has that
# service.  Returns true if so, false otherwise.
sub service_check {
    my ($self, $sunetid, $service) = @_;
    my $sql = "select me_sunetid from membership where me_sunetid = ? and
        me_service = ?";
    my $result = $self->selectrow_array ($sql, undef, $sunetid, $service);
    return defined $result;
}

# Given an account, a service, and a status, set the status of that service
# for that account to the specified status.  Currently, status may be either
# 'active' or 'delete'; eventually, 'frozen' and 'locked' will also be
# supported.  Takes an optional fourth argument specifying the person making
# the change and also handles updating the history table.
#
# If the service is already in the given status, nothing will be done.
# Returns true if a change was made, false otherwise.
sub service_set {
    my ($self, $sunetid, $service, $status, $actor) = @_;
    if ($status eq 'active') {
        my $comment = "Granted service $service";
        return if $self->service_check ($sunetid, $service);
        $self->{AutoCommit} = 0;
        eval {
            my $sql = "insert into membership (me_sunetid, me_service)
                values (?, ?)";
            $self->do ($sql, undef, $sunetid, $service);
            $self->history_log ($sunetid, "Granted service $service", $actor);
            $self->commit;
        };
        if ($@) {
            my $error = $@;
            $self->rollback;
            croak "can't grant service $service for $sunetid: $error";
        }
        $self->{AutoCommit} = 1;
    } elsif ($status eq 'delete') {
        return unless $self->service_check ($sunetid, $service);
        $self->{AutoCommit} = 0;
        eval {
            my $sql = "delete from membership where me_sunetid = ? and
                me_service = ?";
            $self->do ($sql, undef, $sunetid, $service);
            $self->history_log ($sunetid, "Removed service $service", $actor);
            $self->commit;
        };
        if ($@) {
            my $error = $@;
            $self->rollback;
            croak "can't grant service $service for $sunetid: $error";
        }
        $self->{AutoCommit} = 1;
    } else {
        croak "unknown service status $status";
    }
}

##############################################################################
# Module return value and documentation#
#############################################################################

# Make sure the module returns true.
1;

__END__

=head1 NAME

Stanford::LSDB::UserDB - Wrapper module for LS user database connections

=head1 SYNOPSIS

    use Stanford::LSDB::UserDB;

    $dbh = Stanford::LSDB::UserDB->connect;

=head1 DESCRIPTION

This is a subclass of Stanford::LSDB, a small wrapper module around the DBI
and DBD::Oracle modules.  It overrides connect() so that no arguments are
necessary; the connect string, username, and password are determined by the
module.

All other methods work the same as with any DBI object.

This module also provides the following additional methods:

=over 4

=item sqlldr (I<control>, I<log> [, I<extra> ...])

Runs Oracle's B<sqlldr> command with the provided control and log files,
connecting to the user database with the appropriate username and password.
Returns 0 on success and the exit status of B<sqlldr> on failure, where
success is defined as B<sqlldr> exiting with status 0.

=back

Database handles also have the following additional methods:

=over 4

=item history_log (I<principal>, I<message> [, I<actor>])

Add I<message> to the history log for I<principal> with an effective date of
now.  If I<actor> is provided, append " by <actor>" to the end of I<message>
where <actor> is I<actor>.

=item service_check (I<principal>, I<service>)

Check if I<principal> has the service I<service>.  Returns true if it does,
false otherwise.

=item service_set (I<principal>, I<service>, I<status> [, I<actor>])

Set the status of service I<service> for I<principal> to I<status>.
Currently, I<status> should be either "active" or "delete"; eventually,
"frozen" and "locked" will also be supported.  If I<actor> is provided, the
history log will reflect that this change was made by I<actor>.

=back

=head1 AUTHOR

Russ Allbery <rra@stanford.edu>

=head1 SEE ALSO

L<DBI>, L<DBD::Oracle>, L<Stanford::LSDB>

=head1 COPYRIGHT AND LICENSE

Copyright 2002, 2004 Board of Trustees, Leland Stanford Jr. University.
This program is free software; you may redistribute it and/or modify it
under the same terms as Perl itself.

=cut
